@@ -1,9 +1,11 @@
 package KinoSearch::Index::TermBuffer;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
-use base qw( KinoSearch::Util::Class );
+use base qw( KinoSearch::Util::CClass );
 
 our %instance_vars = __PACKAGE__->init_instance_vars(
-    # constructor args
+    # constructor params
     finfos => undef,
 );
 
@@ -12,7 +14,7 @@ sub new {
     $class = ref($class) || $class;
     my %args = ( %instance_vars, @_ );
     verify_args( \%instance_vars, %args );
-    my $self = _new( $class, $args{finfos}, $args{finfos}->get_fnum_map );
+    my $self = _new( $class, $args{finfos}->size );
     return $self;
 }
 
@@ -24,34 +26,14 @@ __XS__
 
 MODULE = KinoSearch    PACKAGE = KinoSearch::Index::TermBuffer
 
-void
-_new(class, finfos_sv, fnum_map_sv) 
-    char       *class;
-    SV         *finfos_sv;
-    SV         *fnum_map_sv;
-PREINIT:
-    TermBuffer *term_buf;
-    char       *ptr;
-    STRLEN      len;
-PPCODE:
-{
-    Kino_New(0, term_buf, 1, TermBuffer);
-    SvREFCNT_inc(finfos_sv);
-    term_buf->finfos = finfos_sv;
-
-    /* reset the TermBuffer */
-    term_buf->termstring = NULL;
-    Kino_TermBuf_reset(term_buf);
-
-    /* copy the fnum_map, derive max_field_num */
-    ptr                     = SvPV(fnum_map_sv, len);
-    term_buf->fnum_map      = Kino_savepvn(ptr, len);
-    term_buf->max_field_num = (len / KINO_FIELD_NUM_LEN) - 1;
-    
-    ST(0) = sv_newmortal();
-    sv_setref_pv(ST(0), class, (void*)term_buf);
-    XSRETURN(1);
-}
+TermBuffer* 
+_new(class, finfos_size) 
+    char *class;
+    I32   finfos_size;
+CODE:
+    /* ignore class */
+    RETVAL = Kino_TermBuf_new(finfos_size);
+OUTPUT: RETVAL
 
 void
 DESTROY(term_buf)
@@ -59,16 +41,6 @@ DESTROY(term_buf)
 PPCODE:
     Kino_TermBuf_destroy(term_buf);
 
-
-void
-_set_text_len(term_buf, new_len)
-    TermBuffer *term_buf;
-    UV          new_len;
-PPCODE:
-    Kino_TermBuf_set_text_len(term_buf, new_len);
-    
-
-
 __H__
 
 #ifndef H_KINOSEARCH_INDEX_TERM_BUFFER
@@ -77,22 +49,23 @@ __H__
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
-#include "KinoSearchUtilMemManager.h"
 #include "KinoSearchIndexTerm.h"
+#include "KinoSearchStoreInStream.h"
+#include "KinoSearchUtilMemManager.h"
 
 typedef struct termbuffer {
-    SV      *finfos;
     char    *termstring;
     STRLEN   text_len;
     STRLEN   capacity;
     char    *fnum_map;
-    U32      max_field_num;
+    I32      max_field_num;
 } TermBuffer;
 
-void Kino_TermBuf_read(TermBuffer*, PerlIO*);
-void Kino_TermBuf_destroy(TermBuffer*);
+TermBuffer* Kino_TermBuf_new(I32);
+void Kino_TermBuf_read(TermBuffer*, InStream*);
 void Kino_TermBuf_reset(TermBuffer*);
 void Kino_TermBuf_set_text_len(TermBuffer*, STRLEN);
+void Kino_TermBuf_destroy(TermBuffer*);
 
 #endif /* include guard */
 
@@ -100,49 +73,57 @@ __C__
 
 #include "KinoSearchIndexTermBuffer.h"
 
+TermBuffer*
+Kino_TermBuf_new(I32 finfos_size) {
+    TermBuffer *term_buf;
+    char       *ptr;
+    STRLEN      len;
+
+    /* allocate */
+    Kino_New(0, term_buf, 1, TermBuffer);
+
+    /* reset the TermBuffer */
+    term_buf->termstring = NULL;
+    Kino_TermBuf_reset(term_buf);
+
+    /* derive max_field_num */
+    term_buf->max_field_num = finfos_size - 1;
+
+    return term_buf;
+}
+
 /* Decode the next term in a term dictionary file (.tii, .tis), but don't turn
  * it into a full-fledged Term object. */
 void
-Kino_TermBuf_read(TermBuffer *term_buf, PerlIO *fh) {
+Kino_TermBuf_read(TermBuffer *term_buf, InStream *instream) {
     I32 text_overlap;
     STRLEN finish_chars_len;
     STRLEN total_text_len;
     U32 field_num;
 
     /* read bytes which are shared between the last term text and this */
-    text_overlap     = Kino_IO_read_vint(fh);
-    finish_chars_len = Kino_IO_read_vint(fh);
+    text_overlap     = instream->read_vint(instream);
+    finish_chars_len = instream->read_vint(instream);
     total_text_len   = text_overlap + finish_chars_len;
     Kino_TermBuf_set_text_len(term_buf, total_text_len);
-    Kino_IO_read_chars(fh, term_buf->termstring, 
+    instream->read_chars(instream, term_buf->termstring, 
         (text_overlap + KINO_FIELD_NUM_LEN),
         finish_chars_len);
 
-    /* read orig field num -- not the same as the final; see FieldInfos */
-    field_num = Kino_IO_read_vint(fh);
+    /* read field num */
+    field_num = instream->read_vint(instream);
     if (field_num > term_buf->max_field_num && field_num != -1)
         Kino_confess("Internal error: field_num %d > max_field_num %d",
             field_num, term_buf->max_field_num);
 
-    if (field_num == -1) {
-        term_buf->termstring[0] = 0xff;     
-        term_buf->termstring[1] = 0xff;
-    }
-    /* assign the KinoSearch-compatible field num from the map */
-    else {
-        Copy( (term_buf->fnum_map + 2*field_num), 
-               term_buf->termstring, 2, char);
-    }
-
+    Kino_encode_bigend_U16( (U16)field_num, term_buf->termstring);
 }
 
 /* Set the TermBuffer object to a sentinel state, indicating that it does not
  * hold a valid Term */
 void
 Kino_TermBuf_reset(TermBuffer *term_buf) {
-    if (term_buf->termstring != NULL) {
-        Kino_Safefree(term_buf->termstring);
-    }
+    Kino_Safefree(term_buf->termstring);
     term_buf->termstring = NULL;
     term_buf->text_len   = 0;
     term_buf->capacity   = 0;
@@ -177,8 +158,6 @@ Kino_TermBuf_set_text_len(TermBuffer *term_buf, STRLEN new_len) {
 void 
 Kino_TermBuf_destroy(TermBuffer *term_buf) {
     Kino_TermBuf_reset(term_buf);
-    Kino_Safefree(term_buf->fnum_map);
-    SvREFCNT_dec(term_buf->finfos);
     Kino_Safefree(term_buf);
 }
 
@@ -202,7 +181,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
