@@ -1,4 +1,6 @@
 package KinoSearch::Search::HitDoc;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
@@ -18,7 +20,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Search::HitDoc - document which matched a Query
+KinoSearch::Search::HitDoc - successful match for a Query
 
 =head1 DESCRIPTION 
 
@@ -30,7 +32,9 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
+
 =cut
+
