@@ -1,22 +1,28 @@
 package KinoSearch::Search::Hits;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
+use KinoSearch::Highlight::Highlighter;
+use KinoSearch::Search::HitCollector;
+
 our %instance_vars = __PACKAGE__->init_instance_vars(
     # params/members
     searcher  => undef,
     query     => undef,
     filter    => undef,
     sort_spec => undef,
+    num_docs  => undef,
 
     # members
-    weight    => undef,
-    hit_queue => undef,
+    weight      => undef,
+    hit_queue   => undef,
+    highlighter => undef,
 
     hit_docs   => undef,
     pointer    => undef,
     total_hits => undef,
-    score_norm => 1,
 
 );
 
@@ -39,30 +45,33 @@ sub seek {
     $self->{pointer} = $start_offset;
 
     # collect enough to satisfy both the offset and the num wanted
-    $num_wanted += $start_offset;
+    my $collector = KinoSearch::Search::HitQueueCollector->new(
+        size => $num_wanted + $start_offset, );
 
     # execute the search!
-    @{$self}{qw( hit_queue total_hits )} = $self->{searcher}->search_hit_queue(
-        num_wanted => $num_wanted,
-        weight     => $self->{weight},
-        filter     => $self->{filter},
-        sort_spec  => $self->{sort_spec},
+    $self->{searcher}->search_hit_collector(
+        hit_collector => $collector,
+        weight        => $self->{weight},
+        filter        => $self->{filter},
+        sort_spec     => $self->{sort_spec},
     );
+    $self->{total_hits} = $collector->get_total_hits;
+    $self->{hit_queue}  = $collector->get_hit_queue;
 
     # turn the HitQueue into HitDocs
-    $self->{hit_docs}   = $self->{hit_queue}->hit_docs;
+    $self->{hit_docs} = $self->{hit_queue}->hit_docs;
 }
 
 sub total_hits {
     my $self = shift;
-    croak("must seek before calling total_hits")
+    $self->seek( 0, 100 )
         unless defined $self->{total_hits};
     return $self->{total_hits};
 }
 
 sub fetch_hit_hashref {
     my $self = shift;
-    croak("must seek before calling fetch_hit_hashref")
+    $self->seek( 0, 100 )
         unless defined $self->{total_hits};
 
     # bail if there aren't any more *captured* hits
@@ -74,9 +83,26 @@ sub fetch_hit_hashref {
         unless defined $hit_doc->get_doc;
     my $hashref = $hit_doc->get_doc()->to_hashref;
 
-    return wantarray
-        ? ( $hashref, $hit_doc->get_score )
-        : $hashref;
+    if ( !exists $hashref->{score} ) {
+        $hashref->{score} = $hit_doc->get_score;
+    }
+    if ( defined $self->{highlighter} and !exists $hashref->{excerpt} ) {
+        $hashref->{excerpt}
+            = $self->{highlighter}->generate_excerpt( $hit_doc->get_doc );
+    }
+
+    return $hashref;
+}
+
+my %create_excerpts_defaults = ( highlighter => undef, );
+
+sub create_excerpts {
+    my $self = shift;
+    verify_args( \%create_excerpts_defaults, @_ );
+    my %args = ( %create_excerpts_defaults, @_ );
+
+    $self->{highlighter} = $args{highlighter};
+    $self->{highlighter}->set_terms( [ $self->{query}->extract_terms ] );
 }
 
 1;
@@ -87,16 +113,17 @@ KinoSearch::Search::Hits - access search results
 
 =head1 SYNOPSIS
 
-    my $hits = $searcher->search($query);
-    $hits->seek(0, 10);
-    my $total_hits = $hits->total_hits;
+    my $hits = $searcher->search( query => $query );
+    $hits->seek( 0, 10 );
     while ( my $hit = $hits->fetch_hit_hashref ) {
-        print "$hit->{title}\n";
+        print "<p>$hit->{title} <em>$hit->{score}</em></p>\n";
     }
 
 =head1 DESCRIPTION
 
-Hits objects are used to access the results of a search.  
+Hits objects are used to access the results of a search.  By default, a hits
+object provides access to the top 100 matches; the seek() method provides
+finer-grained control.
 
 A classic application would be paging through hits.  The first time, seek to a
 START of 0, and retrieve 10 documents.  If the user wants to see more -- and
@@ -111,10 +138,10 @@ more documents.  And so on.
 
 Position the Hits iterator at START, and capture NUM_TO_RETRIEVE docs.
 
-seek I<must> be called before anything else.
-
 =head2 total_hits
 
+    my $num_that_matched = $hits->total_hits;
+
 Return the total number of documents which matched the query used to produce
 the Hits object.  (This number is unlikely to match NUM_TO_RETRIEVE.)
 
@@ -123,15 +150,22 @@ the Hits object.  (This number is unlikely to match NUM_TO_RETRIEVE.)
     while ( my $hit = $hits->fetch_hit_hashref ) {
         # ...
     }
-    
-    # or...
-    while ( my ( $hit, $score ) = $hits->fetch_hit_hashref ) {
-        # ...
-    }
 
-Return the next hit or hit/score pairing.  The hit is retrieved as a hashref,
-with the field names as keys and the field values as values.  In list context,
-fetch_hit_hashref returns the hashref and a floating point score.
+Return the next hit as a hashref, with the field names as keys and the field
+values as values.  An entry for C<score> will also be present, as will an
+entry for C<excerpt> if create_excerpts() was called earlier.  However, if the
+document contains stored fields named "score" or "excerpt", they will not be
+clobbered.
+
+=head2 create_excerpts
+
+    my $highlighter = KinoSearch::Highlight::Highlighter->new(
+        excerpt_field => 'bodytext',    
+    );
+    $hits->create_excerpts( highlighter => $highlighter );
+
+Use the supplied highlighter to generate excerpts.  See
+L<KinoSearch::Highlight::Highlighter|KinoSearch::Highlight::Highlighter>.
 
 =head1 COPYRIGHT
 
@@ -139,7 +173,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =cut
 
